﻿namespace PrintTest
{
  using System;
  using System.Collections.Generic;
  using System.Linq;
  using System.Threading.Tasks;
  using Windows.Foundation;
  using Windows.Graphics.Printing;
  using Windows.Graphics.Printing.OptionDetails;
  using Windows.UI;
  using Windows.UI.Core;
  using Windows.UI.Xaml;
  using Windows.UI.Xaml.Controls;
  using Windows.UI.Xaml.Data;
  using Windows.UI.Xaml.Media;
  using Windows.UI.Xaml.Printing;

  public class ImageData
  {
    public string Title { get; set; }
    public string ImageUri { get; set; }
  }

  public sealed partial class MainPage : Page
  {
    List<ImageData> _imageData;

    public MainPage()
    {
      this.InitializeComponent();

      this._itemsPerPage = 1;

      this.Loaded += OnLoaded;
    }
    void OnLoaded(object sender, RoutedEventArgs e)
    {
      this._imageData = Enumerable.Range(0, 20).Select(
        i => new ImageData()
          {
            Title = string.Format("Image number {0}", i + 1),
            ImageUri = string.Format("ms-appx:///Assets/img.jpg", i)
          }
      ).ToList();
    }
    void OnRegister(object sender, RoutedEventArgs e)
    {
      // Note, this manager can also invoke the print UI for us.
      PrintManager manager = PrintManager.GetForCurrentView();

      manager.PrintTaskRequested += OnPrintTaskRequested;
    }
    void OnPrintTaskRequested(PrintManager sender, PrintTaskRequestedEventArgs args)
    {
      // If I need to be asynchronous, I can get a deferral. I don't *need*
      // to do this here, I'm just faking it.
      var deferral = args.Request.GetDeferral();

      PrintTask printTask = args.Request.CreatePrintTask(
        "My Print Job",
        OnPrintTaskSourceRequestedHandler);

      printTask.Completed += OnPrintTaskCompleted;

      this.AddCustomPrintOption(printTask);

      deferral.Complete();
    }
    void AddCustomPrintOption(PrintTask printTask)
    {
      PrintTaskOptionDetails details = PrintTaskOptionDetails.GetFromPrintTaskOptions(
        printTask.Options);

      // Clear, string "optionId" needs to be in a constant etc.
      PrintCustomItemListOptionDetails formatOptions =
        details.CreateItemListOption("optionId", "Images Per Page");

      var options = new string[] { "1", "4", "9", "16" };

      foreach (var item in options)
      {
        formatOptions.AddItem(item, item);
      }

      details.DisplayedOptions.Add("optionId");

      details.OptionChanged += OnItemsPerPageChanged;
    }
    void OnItemsPerPageChanged(PrintTaskOptionDetails sender, PrintTaskOptionChangedEventArgs args)
    {
      if ((string)args.OptionId == "optionId")
      {
        PrintTaskOptionDetails optionDetails = (PrintTaskOptionDetails)sender;
        string value = optionDetails.Options["optionId"].Value as string;

        if (!string.IsNullOrEmpty(value))
        {
          int itemsPerPage = 0;

          if (int.TryParse(value, out itemsPerPage))
          {
            this.Dispatcher.RunAsync(CoreDispatcherPriority.Normal,
              () =>
              {
                this._pageCount = 0;
                this._itemsPerPage = itemsPerPage;
                this._document.InvalidatePreview();
              }
            );
          }
        }
      }
    }
    void OnPrintTaskCompleted(PrintTask sender, PrintTaskCompletedEventArgs args)
    {
      this._itemsPerPage = 1;
      this._pageCount = 0;
      this._pageSize = null;
      this._imageableRect = null;
      this._document = null;
    }
    async void OnPrintTaskSourceRequestedHandler(PrintTaskSourceRequestedArgs args)
    {
      var deferral = args.GetDeferral();

      await this.Dispatcher.RunAsync(CoreDispatcherPriority.Normal,
        () =>
        {
          this._document = new PrintDocument();

          this._document.Paginate += OnPaginate;
          this._document.GetPreviewPage += OnGetPreviewPage;
          this._document.AddPages += OnAddPages;

          // Tell the caller about it.
          args.SetSource(this._document.DocumentSource);
        }
      );
      deferral.Complete();
    }
    void OnAddPages(object sender, AddPagesEventArgs e)
    {
      // Note: this code does not take notice of any page range specified
      // by the user. It should.
      this.Dispatcher.RunAsync(CoreDispatcherPriority.Normal,
        async () =>
        {
          for (int i = 1; i <= this._pageCount; i++)
          {
            this._document.AddPage(await this.BuildPageAsync(i));
          }
          this._document.AddPagesComplete();
        }
      );
    }
    void OnGetPreviewPage(object sender, GetPreviewPageEventArgs e)
    {
      this.Dispatcher.RunAsync(CoreDispatcherPriority.Normal,
        async () =>
        {
          // NB: No longer caching these in a dictionary.
          UIElement page = await this.BuildPageAsync(e.PageNumber);

          this._document.SetPreviewPage(e.PageNumber, page);
        }
      );
    }
    async Task<UIElement> BuildPageAsync(int pageNumber)
    {
      // Account for pages going 1..N rather than 0..N-1
      int pageIndex = pageNumber - 1;

      // TBD: Unsure about DPI here.
      // Changed from a Canvas in the previous code.
      Grid parentGrid = new Grid();
      parentGrid.Width = this._pageSize.Value.Width;
      parentGrid.Height = this._pageSize.Value.Height;

      // Make a grid
      Grid grid = new Grid();
      grid.Margin = new Thickness(
        this._imageableRect.Value.Left,
        this._imageableRect.Value.Top,
        this._pageSize.Value.Width - this._imageableRect.Value.Width - this._imageableRect.Value.Left,
        this._pageSize.Value.Height - this._imageableRect.Value.Height - this._imageableRect.Value.Top);

      // How many squares?
      int squareCount = (int)(Math.Sqrt(this._itemsPerPage));

      // Make grid rows and cols
      GridLength length = new GridLength(1, GridUnitType.Star);

      for (int i = 0; i < squareCount; i++)
      {
        grid.RowDefinitions.Add(new RowDefinition() { Height = length });
        grid.ColumnDefinitions.Add(new ColumnDefinition() { Width = length });
      }

      // Make items
      int startIndex = this._itemsPerPage * pageIndex;
      int endIndex =
        Math.Min(startIndex + this._itemsPerPage, this._imageData.Count);

      int rowNo = 0;
      int colNo = 0;
      List<Task> imageLoadTasks = new List<Task>();

      for (int itemIndex = startIndex; itemIndex < endIndex; itemIndex++)
      {
        ImageUserControl control = new ImageUserControl();
        Binding binding = new Binding()
        {
          Path = new PropertyPath("ImageUri")
        };
        control.SetBinding(ImageUserControl.ImageUriProperty, binding);
        control.DataContext = this._imageData[itemIndex];

        imageLoadTasks.Add(control.LoadAsync());

        Grid.SetRow(control, rowNo);
        Grid.SetColumn(control, colNo);

        colNo++;

        if (colNo >= squareCount)
        {
          colNo = 0;
          rowNo++;
        }
        grid.Children.Add(control);
      }

      // Offset it into the printable area
      Canvas.SetLeft(grid, this._imageableRect.Value.Left);
      Canvas.SetTop(grid, this._imageableRect.Value.Top);

      parentGrid.Children.Add(grid);

      // Wait for the images to load.
      await Task.WhenAll(imageLoadTasks);

      return (parentGrid);
    }
    void OnPaginate(object sender, PaginateEventArgs e)
    {
      this.Dispatcher.RunAsync(CoreDispatcherPriority.Normal,
        () =>
        {
          this.GetPageSize(e);

          int itemCount = this._imageData.Count;
          this._pageCount = itemCount / this._itemsPerPage;

          if ((itemCount % this._itemsPerPage) > 0)
          {
            this._pageCount++;
          }
          this._document.SetPreviewPageCount(this._pageCount,
            PreviewPageCountType.Final);
        }
      );
    }
    void GetPageSize(PaginateEventArgs e)
    {
      if (this._pageSize == null)
      {
        PrintPageDescription description = e.PrintTaskOptions.GetPageDescription(
          (uint)e.CurrentPreviewPageNumber);

        this._pageSize = description.PageSize;
        this._imageableRect = description.ImageableRect;
      }
    }
    int _itemsPerPage;
    int _pageCount;
    Size? _pageSize;
    Rect? _imageableRect;
    PrintDocument _document;
  }
}